<?php

if( ! defined( 'ABSPATH' ) ) die;

new WPSC_Endpoints();

/**
 * Handle etherscan api queries for block explorer view
 */

class WPSC_Endpoints {

    // prefix name for the transient variable
    const transientPrefix = 'wpsc_';

    const paginationOffset = 25;

    // define endpoints
    function __construct() {

        // get token supply
        add_action( 'rest_api_init', function () {

            register_rest_route( 'wpsc/v1', '/ping/', [
                'methods' => 'GET',
                'callback' => [ $this, 'ping' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/get_tx_contract_account/(?P<network>\d+)/(?P<contract>[a-zA-Z0-9-]+)/(?P<address>[a-zA-Z0-9-]+)/(?P<page>\d+)/(?P<internal>\d+)', [
                'methods' => 'GET',
                'callback' => [ $this, 'getTxAccountInContract' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/get_tx_contract/(?P<network>\d+)/(?P<contract>[a-zA-Z0-9-]+)/(?P<page>\d+)/(?P<internal>\d+)', [
                'methods' => 'GET',
                'callback' => [ $this, 'getTxFromContract' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/total_supply/(?P<network>\d+)/(?P<contract>[a-zA-Z0-9-]+)', [
                'methods' => 'GET',
                'callback' => [ $this, 'getTotalSupply' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/balance/(?P<network>\d+)/(?P<contract>[a-zA-Z0-9-]+)/(?P<address>[a-zA-Z0-9-]+)', [
                'methods' => 'GET',
                'callback' => [ $this, 'getBalance' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/get_tx/(?P<network>\d+)/(?P<contract>[a-zA-Z0-9-]+)/(?P<txid>[a-zA-Z0-9-]+)/(?P<ignore_contract>[0-9-]+)', [
                'methods' => 'GET',
                'callback' => [ $this, 'getTxId' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/remove_cache/', [
                'methods' => 'GET',
                'callback' => [ $this, 'removeCache' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/get_code/(?P<network>\d+)/(?P<contract>[a-zA-Z0-9-]+)', [
                'methods' => 'GET',
                'callback' => [ $this, 'getCode' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/format_float/(?P<float>[0-9\,\.e\+]+)', [
                'methods' => 'GET',
                'callback' => [ $this, 'formatFloat' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/format_float2/(?P<float>[0-9\,\.e\+]+)/(?P<dec>\d+)', [
                'methods' => 'GET',
                'callback' => [ $this, 'formatFloat2' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/nft/(?P<id>\d+)', [
                'methods' => 'GET',
                'callback' => [ $this, 'nft' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/nft1155/(?P<collection_id>\d+)/(?P<id>[a-zA-Z0-9-]+)', [
                'methods' => 'GET',
                'callback' => [ $this, 'nft1155' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/insert-nft/(?P<id>\d+)', [
                'methods' => 'POST',
                'callback' => [ $this, 'nftInsert' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/save-media/', [
                'methods' => 'POST',
                'callback' => [ $this, 'saveMedia' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/retrieve-media-log/', [
                'methods' => 'POST',
                'callback' => [ $this, 'retrieveMedia' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/update-nft-1155/', [
                'methods' => 'POST',
                'callback' => [ $this, 'updateNFT1155' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/nft-erc1155-owners/', [
                'methods' => 'POST',
                'callback' => [ $this, 'NFT1155Owners' ],
                'permission_callback' => '__return_true'
            ]);
            
            register_rest_route( 'wpsc/v1', '/nft-erc1155-my/', [
                'methods' => 'POST',
                'callback' => [ $this, 'NFT1155MyItems' ],
                'permission_callback' => '__return_true'
            ]);
            
            register_rest_route( 'wpsc/v1', '/nft-log/(?P<id>\d+)/(?P<txid>[a-zA-Z0-9-]+)/(?P<to>[a-zA-Z0-9-]+)/(?P<value>[0-9\.]+)', [
                'methods' => 'POST',
                'callback' => [ $this, 'nftLog' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/save-deploy-nft/(?P<id>\d+)', [
                'methods' => 'POST',
                'callback' => [ $this, 'nftSaveDeploy' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/get-nft-by-id/(?P<collid>\d+)/(?P<ids>[0-9-]+)', [
                'methods' => 'POST',
                'callback' => [ $this, 'nftGetByIDs' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/is-minted/(?P<id>\d+)', [
                'methods' => 'POST',
                'callback' => [ $this, 'isMinted' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/nft-ipfs-store', [
                'methods' => 'POST',
                'callback' => [ $this, 'nftIPFSStore' ],
                'permission_callback' => '__return_true'
            ]);

            register_rest_route( 'wpsc/v1', '/nft-exists-ipfs/(?P<id>\d+)', [
                'methods' => 'POST',
                'callback' => [ $this, 'nftExistsIPFS' ],
                'permission_callback' => '__return_true'
            ]);

        });

        // if there is a change in a NFT clear transient endpoint response
        add_action('save_post', function ($post_id) {
            global $post; 
            if (!empty($post) and $post->post_type != 'nft'){
                return;
            }
            $transient_name = "wpsc_nft_" . $post_id;
            delete_transient($transient_name);
        });

    }

    // endpoint callbacks

    public static function ping($params) {
        return new WP_REST_Response(true);
    }

    public static function getTxAccountInContract($params) {
        
        check_ajax_referer('wp_rest', '_wpnonce');
        
        return new WP_REST_Response(
            self::getTx($params['network'], $params['contract'], $params['address'], $params['page'], $params['internal'])
        );
        
    }

    public static function getTxFromContract($params) {

        check_ajax_referer('wp_rest', '_wpnonce');
        
        return new WP_REST_Response(
            self::getTx($params['network'], $params['contract'], null, $params['page'], $params['internal'])
        );

    }

    static private function processInput($input) {

        if ($input) {

            $hashFunction = $type = $from = $to = $value = null;

            $hashFunction = substr($input, 0, 10);
            switch ($hashFunction) {
                case "0xa9059cbb":
                    $type="transfer";
                    $to = "0x" . substr($input, 34, 40);
                    $value = hexdec(substr($input, 75, 63));
                    break;
                case "0x23b872dd":
                    $type="transferFrom";
                    $from = "0x" . substr($input, 34, 40);
                    $to = "0x" . substr($input, 98, 40);
                    $value = hexdec(substr($input, 138));
                    break;
                case "0x095ea7b3":
                    $type="approve";
                    $to = "0x" . substr($input, 34, 40);
                    $value = hexdec(substr($input, 74));
                    break;
                case "0x40c10f19":
                    $type="mint";
                    $to = "0x" . substr($input, 34, 40);
                    $value = hexdec(substr($input, 74));
                    break;
                case "0x42966c68":
                    $type="burn";
                    $value = hexdec(substr($input, 10));
                    break;
                case "0x79cc6790":
                    $type="burnFrom";
                    $from = "0x" . substr($input, 34, 40);
                    $value = hexdec(substr($input, 74));
                    break;
                case "0x8456cb59":
                    $type="pause";
                    break;
                case "0x3f4ba83a":
                    $type="resume";
                    break;
                case "0x983b2d56":
                    $type="addMinter";
                    break;
                case "0x82dc1ec4":
                    $type="addPauser";
                    break;
                case "0x98650275":
                    $type="renounceMinter";
                    break;
                case "0x6ef8d66d":
                    $type="renouncePauser";
                    break;
                case "0x80a70e5f": // raspberry
                case "0xe6c9f1f6": // raspberry wpst
                case "0x45c2e176": // bluemoon
                case "0x3e517ed1": // bluemoon wpst
                case "0x5b060530": // vanilla and pistachio
                case "0x558d4657": // chocolate
                case "0x37d325a1": // vanilla and pistachio wpst
                case "0x95d38e11": // chocolate wpst
                case "0x772d0f3c": // mango
                case "0xc19afa14": // mango wpst
                    $type="contractCreation";
                break;
                case "0xec8ac4d8":
                    $type="icoBuyTokens";
                    break;
                case "0x":
                    $type="icoDirectTransfer";
                    break;
                default:
                    $type = $hashFunction;
                    break;
            }
            if ($value) {
                // convert wei like units to ether like
                $value = $value / 1000000000000000000;
                $value = $value;
            }
            return [$hashFunction, $type, $from, $to, $value];
        } else {
            return [];
        }

    }

    // filter tx based on contract / account address
    private static function getTx($network, $contract, $address, $page, $internal) {

        if (!$subdomain = self::getNetworkSubdomain($network)) {
            return [];
        }

        if (!$page) $page = 1;

        // if we have a transient stored, return it
        if ($txs = self::getTransientResponse($network, $contract, $address, $page, $internal)) {

            return $txs;

        // otherwise hit the Etherscan API
        } else {

            // do we have a key?
            $etherscan_api_key = get_option('etherscan_api_key_option');

            // filter by contract and account
            if ($address) {
                $etherscan_url = 'https://' . $subdomain . '.etherscan.io/api?module=account&action=tokentx&address=' . $address . 
                    '&sort=desc&page=' . $page . '&offset=' . self::paginationOffset . '&apikey=' . trim($etherscan_api_key["api_key"]) . 
                        '&contractAddress=' . $contract;
            // filter by contract
            } else {

                // list internal txs
                if ($internal) {
                    $txlist_endpoint = "txlistinternal";
                    $offset = 10; // internal txs are slower, so lets show less
                // list regular txs
                } else {
                    $txlist_endpoint = "txlist";
                    $offset = self::paginationOffset;
                }
                $etherscan_url = 'https://' . $subdomain . '.etherscan.io/api?module=account&action=' . $txlist_endpoint . '&address=' . $contract . 
                '&page=' . $page . '&offset=' . $offset . '&sort=desc&apikey=' . trim($etherscan_api_key["api_key"]); // we try to use the user api key
            }

//            return $etherscan_url;

            // hit the api
            $response = wp_remote_get( $etherscan_url );

            // successful?
            if ( is_array( $response ) and $response["response"]["code"] == "200") {

                $txlist = json_decode($response["body"], true);

                if (is_array($txlist) and array_key_exists('result', $txlist)) {

                    $txs=[];
                    $localeconv = localeconv();

                    // filter transactions with the contract
                    foreach ($txlist['result'] as $res) {

                        $txs_column = array_column($txs, 'txid');
                        if (array_search($res["hash"], $txs_column)!==false) continue;

                        // parse etherscan input response
                        $hashFunction = '';
                        $to = '';
                        $value = 0;

                        @list($hashFunction, $type, $transferFrom, $to, $value) = self::processInput($res["input"]);

                        // find the date format in settings or set default
                        $settings = WPSCSettingsPage::get();
                        if (!$date_format = $settings["date_format"]) {
                            $date_format = 'Y-m-d';
                        }
                        
                        $from = $res["from"];

                        if ($type=="transferFrom" or $type=="burnFrom") {
                            $from = $transferFrom;
                        }

                        // if this is an internal request then  find internal details
                        if ($internal) {
                            $txs[] = current(self::getTxId([
                                "network"=>$network, 
                                "contract"=>$contract, 
                                "txid"=>$res["hash"],
                                "ignore_contract"=>true // in this case we dont want to filter by contract
                            ]));
                        // otherwise return regular fields
                        } else {

                            // use default to address if not in input
                            if (!$to) {
                                $to = $res["to"];
                            }
                            
                            // ad it to the tx list
                            $txs[] = [
                                'blockNumber' => $res["blockNumber"],
                                'timeStamp' => ($res["timeStamp"])?date($date_format, $res["timeStamp"]):'',
                                'txid' => $res["hash"],
                                'txid_short' => WPSC_helpers::shortify($res["hash"]),
                                'from' => $from,
                                'from_short' => WPSC_helpers::shortify($from),
                                'transfer_from' => $transferFrom,
                                'transfer_from_short' => WPSC_helpers::shortify($transferFrom),
                                'hashFunction' => $hashFunction,
                                'type' => $type,
                                'to' => $to,
                                'to_short' => WPSC_helpers::shortify($to),
                                'value' => $value?WPSC_helpers::formatNumber($value):
                                    WPSC_helpers::formatNumber($res["value"]/1000000000000000000),
                                'isError' => WPSC_helpers::valArrElement($res, 'isError')?$res["isError"]:false,
                                'subdomain' => self::getNetworkSubdomain($network, ''),
                            ];

                        }

                    }

                    // save the wp transient api response
                    if (!empty($txs)) {
                        self::saveTransientResponse($network, $contract, $address, $page, $internal, $txs);
                    }

                    return $txs;
                }

            }

        }

        return [];

    }

    // filter tx based on contract / account address
    public static function getTxId($params) {

        check_ajax_referer('wp_rest', '_wpnonce');

        $network = $params['network'];
        $contract = $params['contract'];
        $txid = $params['txid'];
        $ignore_contract = $params['ignore_contract'];

        if (!$subdomain = self::getNetworkSubdomain($network)) {
            return [];
        }

        // if we have a transient stored, return it
        if ($txs = self::getTransientResponse($network, $txid, null, null, null)) {

            return $txs;

        // otherwise hit the Etherscan API
        } else {

            // do we have a key?
            $etherscan_api_key = get_option('etherscan_api_key_option');

            $etherscan_url = 'https://' . $subdomain . '.etherscan.io/api?module=proxy&action=eth_getTransactionByHash&txhash=' . 
                $txid . '&apikey=' . trim($etherscan_api_key["api_key"]); // we try to use the user api key

            // hit the api
            $response = wp_remote_get( $etherscan_url );

            // successful?
            if ( is_array( $response ) and $response["response"]["code"] == 200) {

                $res = json_decode($response["body"], true);

                if (is_array($res) and array_key_exists('result', $res)) {
                    $res = $res["result"];
                }

                if (is_array($res) and array_key_exists('hash', $res)) {

                    // filtering contract interaction comparing unchecksummed addresses
                    if ($ignore_contract or strtolower($res["to"]) == strtolower($contract)) {

                        // parse etherscan input response
                        $hashFunction = '';
                        $to = '';
                        $type = false;
                        $value = 0;

                        @list($hashFunction, $type, $transferFrom, $to, $value) = self::processInput($res["input"]);

                        // now try to get the block info to get the timestamp
                        $time_stamp = null;
                        if ($res["blockNumber"]) {

                            $etherscan_url = 'https://' . $subdomain . '.etherscan.io/api?module=block&action=getblockreward&blockno=' . 
                                hexdec( substr($res["blockNumber"], 2) ) . '&apikey=' . trim($etherscan_api_key["api_key"]); // we try to use the user api key


                            // hit the api
                            $response = wp_remote_get( $etherscan_url );

                            // successful?
                            if ( is_array( $response ) and $response["response"]["code"] == 200) {

                                $body = json_decode($response["body"], true);
                                $time_stamp = $body["result"]["timeStamp"];

                            }

                        }

                        $txs[] = [
                            'blockNumber' => $res["blockNumber"],
                            'timeStamp' => ($time_stamp)?date('Y-m-d', $time_stamp):'',
                            'txid' => $res["hash"],
                            'txid_short' => WPSC_helpers::shortify($res["hash"]),
                            'from' => $res["from"],
                            'from_short' => WPSC_helpers::shortify($res["from"]),
                            'transfer_from' => $transferFrom,
                            'transfer_from_short' => WPSC_helpers::shortify($transferFrom),
                            'hashFunction' => $hashFunction,
                            'type' => $type,
                            'to' => $to,
                            'to_short' => WPSC_helpers::shortify($to),
                            'value' => $value?WPSC_helpers::formatNumber($value):
                                WPSC_helpers::formatNumber(hexdec($res["value"])/1000000000000000000),
                            'isError' => WPSC_helpers::valArrElement($res, 'isError')?$res["isError"]:false,
                            'subdomain' => self::getNetworkSubdomain($network, ''),
//                            'response' => $res,
                        ];
                    }

                    // save the wp transient api response
                    if (!empty($txs)) {
                        self::saveTransientResponse($network, $txid, null, null, null, $txs);
                    }

                    return $txs;
                }

            }

        }

        return [];

    }

    // filter tx based on contract / account address
    public static function getCode($params) {

        check_ajax_referer('wp_rest', '_wpnonce');

        $network = $params['network'];
        $contract = $params['contract'];

        if (!$subdomain = self::getNetworkSubdomain($network)) {
            return [];
        }

        // if we have a transient stored, return it
        if ($txs = self::getTransientResponse($network, $contract, 'source_code', null, null)) {

            return $txs;

        // otherwise hit the Etherscan API
        } else {

            // do we have a key?
            $etherscan_api_key = get_option('etherscan_api_key_option');

            $etherscan_url = 'https://' . $subdomain . '.etherscan.io/api?module=contract&action=getsourcecode&address=' . 
                $contract . '&apikey=' . trim($etherscan_api_key["api_key"]); // we try to use the user api key

            // hit the api
            $response = wp_remote_get( $etherscan_url );

            // successful?
            if ( is_array( $response ) and $response["response"]["code"] == 200) {

                $res = json_decode($response["body"], true);

                if (is_array($res) and array_key_exists('result', $res)) {
                    self::saveTransientResponse($network, $contract, 'source_code', null, null, $res["result"]);
                    return $res["result"];
                }

            }

        }

        return [];

    }

    // get total token supply
    public static function getTotalSupply($params) {

        check_ajax_referer('wp_rest', '_wpnonce');

        if (!$subdomain = self::getNetworkSubdomain($params['network'])) {
            return [];
        }

        // if we have a transient stored, return it
        if ($supply = self::getTransientResponse($params['network'], $params['contract'], "total_supply", null, null)) {

            return $supply;

        // otherwise hit the Etherscan API
        } else {

            // do we have a key?
            $etherscan_api_key = get_option('etherscan_api_key_option');

            $etherscan_url = 'https://' . $subdomain . '.etherscan.io/api?module=stats&action=tokensupply&contractaddress=' . $params['contract'] . '&apikey=' . trim($etherscan_api_key["api_key"]); // we try to use the user api key

            // hit the api
            $response = wp_remote_get( $etherscan_url );

            // successful?
            if ( is_array( $response ) and $response["response"]["code"] == 200) {

                $supply = json_decode($response["body"], true);

                if (is_array($supply) and WPSC_helpers::valArrElement($supply, 'result')) {

                    $formatted_result = WPSC_helpers::formatNumber($supply["result"] / 1000000000000000000);

                    // save the wp transient api response
                    self::saveTransientResponse($params['network'], $params['contract'], "total_supply", null, null, $formatted_result);

                    return $formatted_result;
                    
                }

            }

        }

        return [];

    }

    // get balance of one holder
    public static function getBalance($params) {

        check_ajax_referer('wp_rest', '_wpnonce');

        if (!$subdomain = self::getNetworkSubdomain($params['network'])) {
            return [];
        }

        // if we have a transient stored, return it
        if ($balance = self::getTransientResponse($params['network'], $params['contract'], $params['address'] . "_balance", null, null)) {

            return $balance;

        // otherwise hit the Etherscan API
        } else {

            // do we have a key?
            $etherscan_api_key = get_option('etherscan_api_key_option');

            $etherscan_url = 'https://' . $subdomain . '.etherscan.io/api?module=account&action=tokenbalance&contractaddress=' . $params['contract'] . '&address=' . $params['address'] . '&tag=latest&apikey=' . trim($etherscan_api_key["api_key"]); // we try to use the user api key

            // hit the api
            $response = wp_remote_get( $etherscan_url );

            // successful?
            if ( is_array( $response ) and $response["response"]["code"] == 200) {

                $balance = json_decode($response["body"], true);

                if (is_array($balance) and array_key_exists('result', $balance)) {

                    $final_balance = WPSC_helpers::formatNumber($balance["result"] / 1000000000000000000);

                    // save the wp transient api response
                    self::saveTransientResponse($params['network'], $params['contract'], $params['address'] . "_balance", null, null, $final_balance);

                    return $final_balance;

                }

            }

        }

        return [];

    }

    // filter tx based on contract / account address
    public static function removeCache() {

        check_ajax_referer('wp_rest', '_wpnonce');
        global $wpdb;

        $current_user = wp_get_current_user();
        if (user_can( $current_user, 'administrator' )) {

            $wpdb->query("DELETE FROM `wp_options` WHERE option_name LIKE '%_transient_" . self::transientPrefix . "%'");
            return new WP_REST_Response(true);

        }

    }

    // format float
    public static function formatFloat($params) {

        check_ajax_referer('wp_rest', '_wpnonce');

        $float = $params['float'];

        if ($float) {
            return WPSC_helpers::formatNumber($float / 1000000000000000000);            
        } else {
            return 0;
        }
        
    }

    // format float
    public static function formatFloat2($params) {

        check_ajax_referer('wp_rest', '_wpnonce');

        $float = $params['float'];

        if ($float) {
            return WPSC_helpers::formatNumber2($float / 1000000000000000000, $params['dec']);
        } else {
            return 0;
        }
        
    }

    // NFT Token URI
    public static function nft($params) {

        $id = $params['id'];

        if (!$id) return;

        $wpsc_nft_id = get_post_meta($id, "wpsc_nft_id", true);

        if (!$wpsc_nft_id) return "Item is not deployed yet";

        $transient_name = "wpsc_nft_" . $id;

        if ($result = get_transient($transient_name)) {
            return $result;
        }

        $res = self::getNFTData($id, $wpsc_nft_id);
        
        set_transient($transient_name, $res);

        return $res;
        
    }

    /**
     * 
     * NFT 1155 Token URI
     * 
     * /wp-json/wpsc/v1/nft1155/{collection-id}/{id}
     * where {collection-id} is a unique ID predefined before deploying the collection and {id} the NFT ID, 
     * 
     */
    public static function nft1155($params) {

        $wpsc_nft_id = $params['id'];
        $collection_id = $params['collection_id'];

        if (!$wpsc_nft_id or !$collection_id) return;

        if (!is_int($wpsc_nft_id)) {
            $wpsc_nft_id = hexdec($wpsc_nft_id);
        }

        // Get NFT WP ID by getting post_id filtering wpsc_item_collection = collection_id and wpsc_nft_id = wpsc_nft_id
        $id = WPSC_Queries::getPostIDByNFTID($collection_id, $wpsc_nft_id);

        if (!is_array($id) or !array_key_exists(0, $id) or !array_key_exists("post_id", $id[0]) or !$id[0]["post_id"]) {
            return "Post ID not found";
        }

        $transient_name = "wpsc_nft1155_" . $id[0]["post_id"];

        if ($result = get_transient($transient_name)) {
            return $result;
        }

        $res = self::getNFTData($id[0]["post_id"], $wpsc_nft_id);
        
        set_transient($transient_name, $res);

        return $res;
        
    }

    static private function getNFTData($id, $wpsc_nft_id) {

        $nft = get_post($id);

        $media_type = get_post_meta($id, "wpsc_media_type", true);
    
        if ( $wpsc_network = get_post_meta($id, 'wpsc_network', true) ) {
            list($color, $icon, $etherscan, $network_val) = WPSC_Metabox::getNetworkInfo($wpsc_network);
        }
        $collection_id = get_post_meta($id, 'wpsc_item_collection', true);
        if ($collection_id) {
            $contract = get_post_meta($collection_id, 'wpsc_contract_address', true);
            $link_etherscan = $etherscan."token/".$contract."?a=".$wpsc_nft_id;
        }
    
        $res = [
            "id"=>$id,
            "name" => $nft->post_title,
            "description" => strip_tags($nft->post_content),
            "attributes" => [
                ["trait_type" => "creator", "value" => "@".get_the_author_meta("display_name", $nft->post_author)],
                ["trait_type" => "last_modified_gmt", "value" => $nft->post_modified_gmt],
            ],
            "media-type" => $media_type,
            "external_url" => get_permalink($id),
            "author_id" => $nft->post_author,
            "author_external_url" => add_query_arg(["a" => $nft->post_author, "id"=>$collection_id], WPSC_assets::getNFTAuthorsPage()),
            "author_avatar" => get_avatar_url($nft->post_author),
            "author_account" => get_post_meta($id, 'wpsc_creator', true),
            "nft_id" => $wpsc_nft_id,
            "network" => $network_val,
            "network_url" => $link_etherscan,
        ];
    
        $media_urls = null;
        if ($media = get_post_meta($id, "wpsc_nft_media_json", true)) {
    
            $tmp = json_decode($media);
            if (is_array($tmp)) {
                foreach ($tmp as $url) {
                    if ($url->id) {
                        if ($ipfs = get_post_meta($url->id, 'wpsc_nft_ipfs', true)) {
                            $media_urls[]=$ipfs;
                        } elseif ($att_url = wp_get_attachment_url($url->id)) {
                            $media_urls[]=$att_url;
                        }
                    }
                }
            }
        }
    
        $cats =  wp_get_post_terms($id, "nft-taxonomy");
        if (is_array($cats)) {
            foreach ($cats as $i => $cat) {
                $ind = $i + 1;
                $res["attributes"][] = [
                    "trait_type" => "Category $ind", 
                    "value" => $cat->name,
                    "link" => add_query_arg("id", $collection_id, get_term_link($cat))
                ];
            }
        }
    
        $attrs =  wp_get_post_terms($id, "nft-tag");
        if (is_array($attrs)) {
            foreach ($attrs as $attr) {
                $res["attributes"][] = [
                    "value" => $attr->name,
                    "link" => add_query_arg("id", $collection_id, get_term_link($attr))
                ];
            }
        }
    
        switch($media_type) {
            case "image":
                if (is_array($media_urls) and array_key_exists(0, $media_urls)) {
                    $res["image"] = $media_urls[0];
                }
                break;
            default:
                if ($thumb = get_the_post_thumbnail_url($id)) {
                    $res["image"] = $thumb;
                }
                if (is_array($media_urls)) {
                    foreach($media_urls as $i=>$url) {
                        if ($i) {
                            $ind = $i + 1;
                            $res["animation_url".$ind] = $url;
                        } else {
                            $res["animation_url"] = $url;
                        }
                    }
                } elseif ($media_urls) {
                    $res["animation_url"] = $media_urls;
                }
                break;
        }

        return $res;
    
    }

    static private function validateAddress($add, $len = 40) {
        return preg_match('/^(0x)?[0-9a-fA-F]{'.$len.'}$/i', $add);
    }

    public static function NFT1155Owners($param) {

        $nft_id = (int) $param['id'];
        $contract = $param["contract"];
        $network_decimal = (int) $param["net"];

        if (!$nft_id or !$contract or !$network_decimal) return new WP_REST_Response("Invalid parameters.", 200 );

        $network_hex = "0x" . dechex($network_decimal);

        $options = get_option('etherscan_api_key_option');
        $nft_moralis_key = 
            (WPSC_helpers::valArrElement($options, "nft_moralis_key") and !empty($options["nft_moralis_key"]))?
                $options["nft_moralis_key"]:
                false;
        if (!$nft_moralis_key) return new WP_REST_Response("Invalid Moralis API key. Are you the system admin? Please setup the Moralis API keys in your WP Smart Contracts settings.", 200 );

        $transient_id = "moralis_owners_".$contract."_".$nft_id."_".$network_hex;

        if ($response = self::getGenericTransientResponse($transient_id)) {
            return new WP_REST_Response($response, 200 );
        }

        $requests_response = Requests::request( 
            'https://deep-index.moralis.io/api/v2/nft/' . $contract . '/' . $nft_id . '/owners', 
            ['accept' => 'application/json', 'X-API-Key' => $nft_moralis_key], 
            [
                "chain"=>$network_hex, 
                "format"=>"decimal"
            ], 
            'GET');

        if ($requests_response->status_code == "200") {
            $result = json_decode($requests_response->body)->result;
            self::saveGenericTransientResponse($transient_id, $result);
            return new WP_REST_Response($result, 200 );
        } else {
            return new WP_REST_Response("An error ocurred: " . json_decode($requests_response->body, true)["message"], 200 );    
        }

    }

    public static function NFT1155MyItems($param) {

        $address = $param['address'];
        $contract = $param["contract"];
        $network_decimal = (int) $param["net"];

        if (!$address or !$contract or !$network_decimal) return new WP_REST_Response("Invalid parameters.", 200 );

        $network_hex = "0x" . dechex($network_decimal);

        $options = get_option('etherscan_api_key_option');
        $nft_moralis_key = 
            (WPSC_helpers::valArrElement($options, "nft_moralis_key") and !empty($options["nft_moralis_key"]))?
                $options["nft_moralis_key"]:
                false;

        if (!$nft_moralis_key) return new WP_REST_Response("Invalid Moralis API key. Are you the system admin? Please setup the Moralis API keys in your WP Smart Contracts settings.", 200 );

        $transient_id = "moralis_my_".$contract."_".$network_hex;

        if (!$response = self::getGenericTransientResponse($transient_id)) {

            $requests_response = Requests::request( 
                'https://deep-index.moralis.io/api/v2/nft/' . $contract . '/owners', 
                ['accept' => 'application/json', 'X-API-Key' => $nft_moralis_key], 
                [
                    "chain"=>$network_hex, 
                    "format"=>"decimal"
                ], 
                'GET');
            if ($requests_response->status_code == "200") {
                $response = json_decode($requests_response->body)->result;
                self::saveGenericTransientResponse($transient_id, $response);
            } else {
                return new WP_REST_Response("An error ocurred: " . json_decode($requests_response->body, true)["message"], 200 );    
            }

        }

        if ($response and is_array($response)) {
            $ids = [];
            foreach ($response as $r) {
                if ($r->owner_of==$address) {
                    $ids[] = $r->token_id;
                }
            }
            return new WP_REST_Response($ids, 200);            
        } else {
            return new WP_REST_Response("An error ocurred", 200 );
        }

    }

    public static function updateNFT1155($param) {

        $user = wp_get_current_user();

        $nickname = get_the_author_meta("display_name", $user->ID );

        $collection_id = (int) $param['collection_id'];
        $contract = $param["contract"];
        $tx = $param["tx"];
        $net_ver = $param["net_ver"];
        $account = $param["account"];
        $blockie_account = $param["blockie_account"];

        $nft_ids = $param["nft_ids"];
        $post_ids = $param["post_ids"];
        $amounts = $param["amounts"];

        $errors = false;

        foreach($post_ids as $index => $post) {

            $post_id = (int) $post;

            if ($error = WPSC_Shortcodes::validateNFTFE($collection_id, $user, $post_id)) {
                $errors[] = $error;
            } else {

                $token_id = $nft_ids[$index]["id"];
                $blockie_id = $nft_ids[$index]["blockie"];
                $supply = $amounts[$index];
        
                // update contract and tx data
                update_post_meta($post_id, "wpsc_collection_contract", $contract);
                update_post_meta($post_id, "wpsc_txid", $tx);
                update_post_meta($post_id, "wpsc_nft_id", $token_id);
                update_post_meta($post_id, "wpsc_creator", $account);
                update_post_meta($post_id, "wpsc_network", $net_ver);
                update_post_meta($post_id, "wpsc_nft_url", get_rest_url(null, "wpsc/v1/nft1155/" . $collection_id . "/" . $token_id));
                update_post_meta($post_id, "wpsc_creator_blockie", $blockie_account);
                update_post_meta($post_id, "wpsc_nft_id_blockie", $blockie_id);

                // publish the post
                wp_update_post(['ID' => $post_id, 'post_status' => 'publish']);

            }
    
        }
        
        return new WP_REST_Response($errors, 200 );

    }

    // endpoint callback to create NFT on the interface
    public static function nftInsert($param) {

        $user = wp_get_current_user();

        $nickname = get_the_author_meta("display_name", $user->ID );

        // sanitize integer
        @$nft_id = (int) $param['nft_id'];

        $collection_id = (int) $param['collection_id'];

        if ($error = WPSC_Shortcodes::validateNFTFE($collection_id, $user, $nft_id)) {
            return new WP_REST_Response($error , 200 );
        }

        // sanitize texts
        $title = sanitize_text_field($param['title']);
        if (!$title) return new WP_REST_Response("Title is not valid", 200);

        // sanitize integer
        @$nft_id = (int) $param['nft_id'];

        $collection_id = (int) $param['collection_id'];

        if ($error = WPSC_Shortcodes::validateNFTFE($collection_id, $user, $nft_id)) {
            return new WP_REST_Response($error , 200 );
        }

        // sanitize texts
        $title = sanitize_text_field($param['title']);
        if (!$title) return new WP_REST_Response("Title is not valid", 200);

        @$tainted_custom_atts = $param['custom_atts'];
        @$tainted_custom_tax = $param['custom_tax'];

        @$owner = sanitize_text_field($param['owner']);

        @$owner = sanitize_text_field($param['owner']);
        @$skip_owner = sanitize_text_field($param['skip_owner']);
        if (!$skip_owner and !self::validateAddress($owner))
            return new WP_REST_Response("The recipient is not a valid address", 200);

        @$supply = (int) sanitize_text_field($param['supply']);
        if (!$supply) $supply = 0;

        // if it is adding the NFT and is an ERC1155 then validate the qty
        if (!$nft_id) {
            $flavor = get_post_meta($collection_id, "wpsc_flavor", true);
            if (($flavor=="yuzu" or $flavor=="ikasumi") and !$supply) {
                return new WP_REST_Response("Supply is not valid, please write a positive number", 200);
            }
        }
        
        // sanitize media json string
        $media = $param['media'];
        if (is_null(json_decode($media))) {
            $media = "";
        }
        $media_type = sanitize_text_field($param['media_type']);
        if (!in_array($media_type, ["image", "video", "audio", "document", "3dmodel"])) {
            $media_type="";
        }
        
        // sanitize textarea keeping the line breaks
        $description = $param['description'];
        if ($description) {
            $description = implode( "\n", array_map( 'sanitize_textarea_field', explode( "\n", $description ) ) );
        }
        
        // sanitize numeric arrays
        $tags = self::integrify($param['tags'], $tainted_custom_atts, "nft-tag");
        $categories = self::integrify($param['categories'], $tainted_custom_tax, "nft-taxonomy");

        $status = 'publish';
        if ($param["draft"]) {
            $status = "draft";
        }

        $arr = [
            'ID' => $nft_id,
            'post_author' => $user->ID,
            'post_content' => $description,
            'post_title' => $title,
            'post_status' => $status,
            'post_type' => 'nft'
        ];

        @$media_id = json_decode($media, true)[0]["id"];

        $the_id = false;

        if ($media_id) {
            $the_id = WPSC_Queries::getPostIDByMediaAndCollection($collection_id, $media_id);
        }

        if (!$the_id) {
            $the_id = wp_insert_post($arr);
        } else {
            // update fields
            wp_update_post([
                'ID' => $the_id,
                'post_title' => $title,
                'post_content' => $description
            ]);
        }

        if ($the_id) {

            update_post_meta($the_id, "wpsc_nft_owner", $owner);
            update_post_meta($the_id, "wpsc_nft_supply", $supply);
            update_post_meta($the_id, "wpsc_item_collection", $collection_id);
            update_post_meta($the_id, "wpsc_nft_media_json", $media);
            update_post_meta($the_id, "original_author", $nickname);
            update_post_meta($the_id, "original_author_id", $user->ID);
            update_post_meta($the_id, "wpsc_media_type", $media_type);
            
            if ($param["batch"]) {
                update_post_meta($the_id, "wpsc_is_batch", true);
            }
            
            wp_set_object_terms( $the_id, $categories, 'nft-taxonomy', false );
            wp_set_object_terms( $the_id, $tags, 'nft-tag', false );

            // clear cache
            delete_transient("wpsc_nft_" . $nft_id);

            if ($param['log_id']) {
                if (self::updateTransientDataPost($param['log_id'], $param['log_index'], $the_id) === false) {
                    error_log("semaphore failed for some reason");
                    return new WP_REST_Response("An error ocurred inserting the NFT", 200 );
                }
            }

            return new WP_REST_Response($the_id, 200 );
        } else {
            return new WP_REST_Response("An error ocurred inserting the NFT", 200 );
        }

    }

    private static function upload($file) {

        $filename = basename($file);

        $content = false;
        if (substr(strtolower($file), 0, 4) == "http") {
            $file = str_replace('\\/', '/', $file);
            $response = wp_remote_get( $file ); 
            if ( is_array( $response ) && ! is_wp_error( $response ) && wp_remote_retrieve_response_code($response) == "200") {
                $content = $response['body'];
            }
        } else {
            $content = file_get_contents($file);
        }

        if ($content) {

            if ($id = WPSC_Queries::getMediaByHash($content)) {
                return $id;
            }

            $upload_file = wp_upload_bits($filename, null, $content);
            if (!$upload_file['error']) {
                $wp_filetype = wp_check_filetype($filename, null );
                $attachment = array(
                    'post_mime_type' => $wp_filetype['type'],
                    'post_title' => preg_replace('/\.[^.]+$/', '', $filename),
                    'post_content' => '',
                    'post_status' => 'inherit'
                );
                $attachment_id = wp_insert_attachment( $attachment, $upload_file['file'] );
                if (!is_wp_error($attachment_id)) {
                    require_once(ABSPATH . "wp-admin" . '/includes/image.php');
                    require_once(ABSPATH . "wp-admin" . '/includes/media.php');
                    $attachment_data = wp_generate_attachment_metadata( $attachment_id, $upload_file['file'] );
                    wp_update_attachment_metadata( $attachment_id,  $attachment_data );
                    update_post_meta($attachment_id, "wpsc_hash", md5($content));
                }
                return $attachment_id;
            }
    
        }

        return false;

    }

    private static function appendTransient($unique_id, $message) {
        set_transient($unique_id, get_transient($unique_id) . $message . "<br/>", 24*60*60);
    }

    private static function updateTransientData($unique_id, $data_index, $media_id, $media_url, $media_mime) {        
        $transient_id = $unique_id . "_data";
        if ($data = get_transient($transient_id) and is_array($data)) {
            $data[$data_index]["media_id"] = $media_id;
            $data[$data_index]["media_url"] = $media_url;
            $data[$data_index]["media_mime"] = $media_mime;
            set_transient($transient_id, $data, 24*60*60);
        }
    }

    private static function semaphoreGetFilename($id) {
        $upload_dir = wp_upload_dir();
        return $upload_dir['basedir'] . '/wpsc-semaphore-'.$id.'.txt';
    }

    private static function semaphoreInit($id) {
        $file = self::semaphoreGetFilename($id);
        if (!file_exists($file)) {
            if (file_put_contents($file, "semaphore")===false) return false;
        }
        return file_exists($file);
    }

    private static function semaphoreWait($id) {

        if (!self::semaphoreInit($id)) {
            return false;
        }
        
        $filename = self::semaphoreGetFilename($id);
        
        $handle = fopen($filename, 'w');
        
        if ($handle===false) {
            return false;
        }
        
        $attemp = 0;
        
        while (!flock($handle, LOCK_EX)) {
            usleep(500);
            $attemp++;
            if ($attemp==40) {
                return false;
            }
        }
        
        return $handle;

    }
    
    private static function SemaphoreSignal($handle) {
        fclose($handle);
    }

    private static function updateTransientDataPost($unique_id, $data_index, $post_id) {
        
        $handle = self::semaphoreWait(2);
        if ($handle===false) return false;

        $transient_id = $unique_id . "_data";
        if ($data = get_transient($transient_id) and is_array($data)) {
            $data[$data_index]["post_id"] = $post_id;
            set_transient($transient_id, $data, 24*60*60);
        }

        self::SemaphoreSignal($handle);

    }

    private static function returnMediaData($id, $media, $unique_id, $message, $index, $ipfs=false) {

        $ret_message = '';
        if (is_numeric($id)) {
            if ($url = wp_get_attachment_url($id)) {
                if (!empty($url)) {

                    if ($ipfs) {
                        $res = self::nftIPFSStore(["attachment_id"=>$id]);
                        if (is_object($res) and property_exists($res, "data")) {
                            if (substr($res->data, 0, 8) == "SUCCESS,") {
                                $url = substr($res->data, 8);
                            } elseif (substr($res->data, 0, 8) == "https://") {
                                $url = $res->data;
                            }
                        }
                    }

                    // get id, url and mime type
                    $mime = get_post_mime_type($id);

                    if ($unique_id) {
                        $ret_message = "Media: " . $media . " ($id) " . $message . " " . $url;
                        self::appendTransient($unique_id, $ret_message);
                        self::updateTransientData($unique_id, $index, $id, $url, $mime);
                    }

                    return [["id"=>$id, "url"=>$url, "mime"=>$mime, "message"=>$ret_message]];
                }
            }
        }

        $message = "<span style=\"color: red\">An error ocurred processing media file: " . $media . "</span>";
        self::appendTransient($unique_id, $message);
        return [["message"=>$message]];

    }

    public static function retrieveMedia($param) {
        $unique_id = $param["log_id"];
        if (substr($unique_id, 0, 5)=="wpsc_") {
            return new WP_REST_Response(get_transient($unique_id), 200 );
        }
    }

    public static function saveMedia($param) {

        $handle = self::semaphoreWait(1);
        if ($handle===false) {
            return new WP_REST_Response([["message"=>"<span style=\"color: red\">An error ocurred processing media file: " . $media . "</span>"]], 200 );
        }

        $media = $param["media"];
        $unique_id = $param["log_id"];
        $index = $param["index"];
        $ipfs = $param["ipfs"];
        $num = intval($media);
        if ($num>0) {
            // is a media ID, find the URL
            return new WP_REST_Response(
                self::returnMediaData($num, $media, $unique_id, "Image ID processed", $index, $ipfs), 
            200);
        } elseif (substr(strtolower($media), 0, 4) == "http") {
            // is an URL
            if ($ret_id = self::upload($media)) {
                return new WP_REST_Response(
                    self::returnMediaData($ret_id, $media, $unique_id, "URL downloaded", $index, $ipfs), 
                200);    
            }
        } elseif (preg_match('/^[\/\w\-. ]+$/', $media)) {
            // is a media in /uploads/wpsc ?
            $filename = wp_upload_dir()["basedir"] . '/wpsc/' . $media;
            if (file_exists($filename)) {
                if ($id = self::upload($filename)) {
                    return new WP_REST_Response(
                        self::returnMediaData($id, $media, $unique_id, "Local file processed", $index, $ipfs),
                    200);    
                }
            }
        }
        if ($unique_id) {
            self::appendTransient($unique_id, "<span style=\"color: red\">An error ocurred processing media file: " . $media . "</span>");
        }

        self::SemaphoreSignal($handle);

        return new WP_REST_Response([["message"=>"<span style=\"color: red\">An error ocurred processing media file: " . $media . "</span>"]], 200 );
    }

    public static function nftLog($param) {

        $post_id = (int) $param['id'];
        $txid = sanitize_text_field($param['txid']);
        $to = sanitize_text_field($param['to']);
        if (is_numeric($param['value'])) {
            $value = $param['value'];
        } else {
            $value = 0;
        }
        $date = date("Y-m-d H:i:s");
        if (!$post_id or !$value) return new WP_REST_Response("Invalid values" , 200 );
        if (!self::validateAddress($to) or !self::validateAddress($txid, 64)) return new WP_REST_Response("Invalid data" , 200 );
        if (get_post_type($post_id)!="nft") return new WP_REST_Response("Invalid post" , 200 );

        $log_history = json_decode(get_post_meta($post_id, 'wpsc_log_history', true), true);
        if (empty($log_history)) {
            $log_history = [];
        }
        $log_history[] = ["txid"=>$txid, "to"=>$to, "value"=>$value, "date"=>$date];
        update_post_meta($post_id, "wpsc_log_history", json_encode($log_history));

        return json_encode("done");

    }
    
    public static function isMinted($param) {

        $user = wp_get_current_user();

        // sanitize integer
        $nft_id = (int) $param['id'];

        $nft_id_check = (int) get_post_meta($nft_id, 'wpsc_nft_id', true);

        if ($nft_id_check>0) {
            return new WP_REST_Response("true" , 200 );
        } else {
            return new WP_REST_Response("false" , 200 );
        }

    }

    public static function nftGetByIDs($param) {

        // sanitize integer
        $collection_id = (int) $param['collid'];

        $contract = get_post_meta($collection_id, 'wpsc_contract_address', true);

        if (!$contract) return "Collection is not deployed";
        if (!$collection_id) return "Invalid collection ID";
		if (get_post_type($collection_id)!="nft-collection") return "Invalid collection ID";

        $arr_unsanitized = explode("-", $param['ids']);

        $nfts = '';

        if (is_array($arr_unsanitized)) {
            foreach($arr_unsanitized as $elem_unsanitized) {
                $elem = (int) $elem_unsanitized;
                if ($elem) {
                    if ($nfts) $nfts .= ",";
                    $nfts .= $elem;
                }
            }
        }

        if (!empty($nfts) and !empty($contract)) {

            $transient_id = $contract."_".$nfts;
            if ($response = self::getGenericTransientResponse($transient_id)) return new WP_REST_Response($response, 200 );
            
            if ($response = WPSC_Queries::getNFTsByID($nfts, $contract)) {
                $token_uris = [];
                if (is_array($response)) {
                    foreach($response as $p) {
                        $token_uris[] = [
                            "nft" => self::nft(["id"=>$p["ID"]]),
                            "json" => get_post_meta($p["ID"], 'wpsc_nft_media_json', true),
                            "media_type" => get_post_meta($p["ID"], 'wpsc_media_type', true),
                            "ID" => $p["ID"]
                        ];
                    }
                }
                self::saveGenericTransientResponse($transient_id, $token_uris);
                return new WP_REST_Response($token_uris, 200);
            }

        }

        return new WP_REST_Response([], 200 );

    }

    public static function nftExistsIPFS($param) {
        
        // get attachment ID
        $attachment_id = (int) $param['id'];
        if (!$attachment_id or get_post_type($attachment_id)!="attachment") return new WP_REST_Response(false, 200 );

        // verify if the attachment was deployed already
        $wpsc_nft_ipfs = get_post_meta($attachment_id, 'wpsc_nft_ipfs', true);

        if ($wpsc_nft_ipfs) return new WP_REST_Response(true , 200 );

        return new WP_REST_Response(false , 200 );

    }

    public static function nftIPFSStore($param) {
        
        // get attachment ID
        $attachment_id = (int) $param['attachment_id'];
        if (!$attachment_id or get_post_type($attachment_id)!="attachment") return new WP_REST_Response("Invalid attachment", 200 );

        // verify if the attachment was deployed already
        $wpsc_nft_ipfs = get_post_meta($attachment_id, 'wpsc_nft_ipfs', true);
        if ($wpsc_nft_ipfs) return new WP_REST_Response($wpsc_nft_ipfs , 200 );

        // get nft storage api key
        $options = get_option('etherscan_api_key_option');
        $nft_storage_key = (WPSC_helpers::valArrElement($options, "nft_storage_key") and !empty($options["nft_storage_key"]))?$options["nft_storage_key"]:false;
        if (!$nft_storage_key) return new WP_REST_Response("Invalid NFT storage key. Are you the system admin? Please setup the nft.storage keys in your WP Smart Contracts settings.", 200 );

        // get file content
        $file = file_get_contents( get_attached_file( $attachment_id ) );
        if (!$file) return new WP_REST_Response("Invalid attachment", 200 );

        // call the NFT Storage endpoint
        $api = new \RestClient( [ 'base_url' => 'https://api.nft.storage' ] );
        $result = $api->post( '/upload', $file, ['Authorization' => 'Bearer ' . $nft_storage_key] );

        if ( 200 === $result->info->http_code ) {
            $wpsc_nft_ipfs = "https://ipfs.io/ipfs/" . $result->decode_response()->value->cid;

            // store this on the attachment post
            update_post_meta($attachment_id, 'wpsc_nft_ipfs', $wpsc_nft_ipfs);

            // clear transient cache for NFT endpoints
            WPSC_Queries::clearNFTTokenURI();

            return new WP_REST_Response("SUCCESS,".$wpsc_nft_ipfs, 200 );
        } else {
            return new WP_REST_Response($result->error, 200 );
        }

    }
    
    // endpoint callback to create NFT on the interface
    public static function nftSaveDeploy($param) {

        $user = wp_get_current_user();

        // sanitize integer
        $nft_id = (int) $param['nft_id'];

        $collection_id = (int) $param['wpsc-item-collection'];
        if ($error = WPSC_Shortcodes::validateNFTFE($collection_id, $user, $nft_id)) {
            return new WP_REST_Response($error , 200 );
        }

        $nft_id_check = (int) get_post_meta($nft_id, 'wpsc_nft_id', true);

        if ($nft_id_check>0) {
            return new WP_REST_Response("Item was already minted" , 200 );
        }

        $wpsc_collection_contract = WPSC_Metabox::cleanUpText($param["wpsc-collection-contract"]);
        update_post_meta($nft_id, 'wpsc_collection_contract', $wpsc_collection_contract);
    
        $wpsc_item_collection = WPSC_Metabox::cleanUpText($param["wpsc-item-collection"]);
        update_post_meta($nft_id, 'wpsc_item_collection', $wpsc_item_collection);
    
        $wpsc_network = WPSC_Metabox::cleanUpText($param["wpsc-network"]);
        update_post_meta($nft_id, 'wpsc_network', $wpsc_network);
    
        $wpsc_txid = WPSC_Metabox::cleanUpText($param["wpsc-txid"]);
        update_post_meta($nft_id, 'wpsc_txid', $wpsc_txid);
    
        $wpsc_creator = WPSC_Metabox::cleanUpText($param["wpsc-creator"]);
        update_post_meta($nft_id, 'wpsc_creator', $wpsc_creator);
    
        $wpsc_creator_blockie = WPSC_Metabox::cleanUpText($param["wpsc-creator-blockie"]);
        update_post_meta($nft_id, 'wpsc_creator_blockie', $wpsc_creator_blockie);
    
        $wpsc_nft_id = WPSC_Metabox::cleanUpText($param["wpsc-nft-id"]);
        update_post_meta($nft_id, 'wpsc_nft_id', $wpsc_nft_id);
    
        $wpsc_nft_id_blockie = WPSC_Metabox::cleanUpText($param["wpsc-nft-id-blockie"]);
        update_post_meta($nft_id, 'wpsc_nft_id_blockie', $wpsc_nft_id_blockie);
    
        $wpsc_nft_url = WPSC_Metabox::cleanUpText($param["wpsc-nft-url"]);
        update_post_meta($nft_id, 'wpsc_nft_url', $wpsc_nft_url);

        $wpsc_nft_supply = WPSC_Metabox::cleanUpText($param["wpsc-nft-supply"]);
        update_post_meta($nft_id, 'wpsc_nft_supply', $wpsc_nft_supply);

        return new WP_REST_Response(get_permalink($nft_id), 200 );

    }

    static private function getNetworkSubdomain($network, $prefix="api") {

        // translate network

        if ( $arr = WPSC_helpers::getNetworks() ) {

            if ($network==1) {
                return $prefix;
            } else {
                if ($prefix) return $prefix . '-' . $arr[$network]["name"];
                else  return $arr[$network]["name"];
            }

        }

    }

    // get txs stored in wp transient
    private static function getTransientResponse($network, $contract, $address, $page, $internal) {

        // cache is activated?
        if (!$expiration_time = WPSCSettingsPage::get('expiration_time')) {
            return false;
        }

        $transient_name = self::transientPrefix . $network . "_" . $contract . "_" . $address . "_" . $page . "_" . $internal;

        if ($t = get_transient($transient_name)) {
            return $t;
        } else {
            return false;
        }

    }

    // store txs to wp transient
    private static function saveTransientResponse($network, $contract, $address, $page, $internal, $txs) {

        // cache is activated?
        if (!$expiration_time = WPSCSettingsPage::get('expiration_time')) {
            return false;
        }

        $transient_name = self::transientPrefix . $network . "_" . $contract . "_" . $address . "_" . $page . "_" . $internal;

        set_transient($transient_name, $txs, $expiration_time);

    }

    private static function getGenericTransientResponse($id) {

        // cache is activated?
        if (!$expiration_time = WPSCSettingsPage::get('expiration_time')) {
            return false;
        }

        $transient_name = self::transientPrefix . $id;

        if ($t = get_transient($transient_name)) {
            return $t;
        } else {
            return false;
        }

    }

    // store txs to wp transient
    private static function saveGenericTransientResponse($id, $content) {

        // cache is activated?
        if (!$expiration_time = WPSCSettingsPage::get('expiration_time')) {
            return false;
        }

        $transient_name = self::transientPrefix . $id;

        set_transient($transient_name, $content, $expiration_time);

    }

    static public function integrify($arr, $tainted_custom_atts=false, $taxonomy="nft-tag") {
        
        if (!$tainted_custom_atts and !is_array($arr)) return [];

        $new = [];

        if ($tainted_custom_atts) {

            if (is_array($tainted_custom_atts)) {
                $arr_atts = $tainted_custom_atts;
            } else {
                $arr_atts = explode(",", $tainted_custom_atts);
            }
            if (is_array($arr_atts)) {

                foreach($arr_atts as $i => $att) {

                    $sanitized_att = sanitize_text_field($att);
                    $sanitized_att = trim($att);

                    if ($att and strlen($sanitized_att)<30) {
                        if ($res = term_exists($sanitized_att, $taxonomy)) {
                            if ($term_id = $res["term_id"] and array_search($term_id, $new)===false) {
                                $new[] = (int) $term_id;
                            }
                        } else {
                            $res = wp_insert_term($sanitized_att, $taxonomy);
                            if (!is_wp_error($res) and $term_id = $res["term_id"] and array_search($term_id, $new)===false) {
                                $new[] = (int) $term_id;
                            }
                        }
                    }
                    if ($i==19) {
                        break;
                    }
                }
                 
            }
        }

        if (is_array($arr)) {
            foreach($arr as $i) {
                $j = (int) $i;
                if ($j and array_search($j, $new)===false) {
                    $new[]=$j;
                }
            }    
        }

        return $new;

    }

}
