<?php

if( ! defined( 'ABSPATH' ) ) die;

new WPSC_Queries();

/**
 * Handle etherscan api queries for block explorer view
 */

class WPSC_Queries {

    static public function nftAuthors() {

        global $wpdb;

        $authors = $wpdb->get_results("SELECT DISTINCT post_author FROM $wpdb->posts WHERE post_status=\"publish\" AND post_type=\"nft\"", ARRAY_A);
    
        if (!empty($authors)) {
            $authors = array_column($authors, 'post_author');
            return $wpdb->get_results("SELECT user_nicename FROM $wpdb->users WHERE ID in (".implode(",", $authors).")", ARRAY_A);
        }

        return [];

    }

    /**
     * Get deployed ERC-1155 Collections
     */
    static public function nftERC1155Collections($id=false) {
        $tmp = self::nftCollections(false, true, $id);
        $res = [];
        $networks = WPSC_helpers::getNetworks();
        if (is_array($tmp)) {
            foreach($tmp as $rec) {
                if (WPSC_helpers::valArrElement($rec, 'erc1155') and $rec["erc1155"] and 
                    WPSC_helpers::valArrElement($rec, 'deployed') and $rec["deployed"]) {
                    $rec["network_name"] = $networks[$rec["network"]]["title"];
                    $rec["network_color"] = $networks[$rec["network"]]["color"];
                    $res[] = $rec;
                }
            }
        }
        return $res;
    }

    static public function nftCollections($selected=false, $get_deployed=false, $id=false) {
        global $wpdb;
        $cond = "";
        if ($id) {
            $cond = " AND ID=$id ";
        }
        $res = $wpdb->get_results("SELECT ID, post_title FROM $wpdb->posts WHERE post_status=\"publish\" AND post_type=\"nft-collection\" $cond ORDER BY post_title", ARRAY_A);
        if (is_array($res) and ($selected or $get_deployed)) {
            foreach($res as $i => $row) {
                if ($row["ID"]==$selected) {
                    $res[$i]["selected"] = true;
                    if (!$get_deployed) {
                        return $res;
                    }
                }
                if (empty($res[$i]["post_title"])) {
                    $res[$i]["post_title"] = "Untitled";
                }
                if ($get_deployed) {
                    if ($contract = get_post_meta($row["ID"], 'wpsc_contract_address', true)) {
                        $res[$i]["deployed"] = $contract;
                    }
                    if ($net = get_post_meta($row["ID"], 'wpsc_network', true)) {
                        $res[$i]["network"] = $net;
                    }
                    if ($flavor = get_post_meta($row["ID"], 'wpsc_flavor', true) and ($flavor=="yuzu" or $flavor=="ikasumi")) {
                        $res[$i]["erc1155"] = true;
                    }
                }
            }
        }
        return $res;
    }

    static public function getTaxonomy($tax) {
        global $wpdb;
        return $wpdb->get_results("SELECT t.term_id, t.name FROM $wpdb->term_taxonomy tax, $wpdb->terms t WHERE tax.term_id=t.term_id AND taxonomy=\"".$tax."\" ORDER BY t.name", ARRAY_A);
    }

    static public function getMediaByHash($content) {

        global $wpdb;

        $sql = "
            SELECT p.ID 
            FROM wp_posts p, wp_postmeta m 
            WHERE p.post_type=\"attachment\" AND 
                  p.ID=m.post_id AND 
                  m.meta_key=\"wpsc_hash\" AND 
                  m.meta_value=\"".md5($content)."\"
        ";

        $res = $wpdb->get_results($sql, ARRAY_A);

        if (is_array($res)) {
            return @$res[0]["ID"];
        }

        return false;

    }

    static public function getNFTsByID($ids, $contract) {

        $str = str_replace(',', '', $ids);
        if (!ctype_digit($str)) return;

        if (!preg_match('/^(0x)?[0-9a-fA-F]{40}$/', $contract, $output_array)) return;

        global $wpdb;

        $sql = "
        SELECT p.ID
        FROM $wpdb->posts p, $wpdb->postmeta m, $wpdb->postmeta m2
        WHERE 
            p.post_type=\"nft\" AND 
            p.post_status=\"publish\" AND 
            p.ID=m.post_id AND 
            p.ID=m2.post_id AND 
            m.meta_key=\"wpsc_nft_id\" AND 
            m.meta_value IN ($ids) AND 
            m2.meta_key=\"wpsc_collection_contract\" AND 
            m2.meta_value=\"$contract\"";

        $res = $wpdb->get_results($sql, ARRAY_A);
        return $res;

    }

    // Get NFT WP ID by getting post_id filtering wpsc_item_collection = collection_id and wpsc_nft_id = wpsc_nft_id
    static function getPostIDByNFTID($collection_id, $wpsc_nft_id) {

        global $wpdb;

        $collection_id = (int) $collection_id;
        $wpsc_nft_id = (int) $wpsc_nft_id;
        
        $sql = "
        SELECT  nft.post_id 
        FROM    $wpdb->postmeta collection, 
                $wpdb->postmeta nft 
        WHERE   collection.post_id=nft.post_id AND 
                collection.meta_key=\"wpsc_item_collection\" AND 
                collection.meta_value=\"$collection_id\" AND 
                nft.meta_key=\"wpsc_nft_id\" AND 
                nft.meta_value=\"$wpsc_nft_id\"
        ";

        $res = $wpdb->get_results($sql, ARRAY_A);
        return $res;

    }

    static public function getNFTIdsByAuthor($collection_id, $author_id) {

        $collection_id = (int) $collection_id;
        if ($collection_id==0) return [];

        $author_id = (int) $author_id;
        if ($author_id==0) return [];

        global $wpdb;
        return $wpdb->get_results("
        SELECT m2.meta_value as nft_id 
        FROM $wpdb->postmeta m, 
        $wpdb->postmeta m2,
        $wpdb->posts p
        WHERE m.post_id=m2.post_id AND 
        m.meta_value=$collection_id AND 
        p.ID=m.post_id AND
        m.meta_key=\"wpsc_item_collection\" AND 
        m2.meta_key=\"wpsc_nft_id\" AND
        p.post_author=$author_id AND
        p.post_status=\"publish\"", ARRAY_A);
        
    }

    static public function getNFTIds($collection_id) {

        $collection_id = (int) $collection_id;

        if ($collection_id==0) return [];

        global $wpdb;
        return $wpdb->get_results("
        SELECT m2.meta_value as nft_id 
        FROM $wpdb->postmeta m, 
        $wpdb->postmeta m2,
        $wpdb->posts p
        WHERE m.post_id=m2.post_id AND 
        p.ID=m.post_id AND
        m.meta_value=$collection_id AND 
        m.meta_key=\"wpsc_item_collection\" AND 
        m2.meta_key=\"wpsc_nft_id\" AND
        p.post_status=\"publish\"", ARRAY_A);
    }

    static public function clearNFTTokenURI() {
        global $wpdb;
        $wpdb->query("DELETE FROM `wp_options` WHERE option_name LIKE '%_transient_wpsc_nft_%'");
    }

    static public function getPostIDByMediaAndCollection($collection_id, $media_id) {
        global $wpdb;
        $res = $wpdb->get_results("
        SELECT  p.ID 
        FROM    $wpdb->postmeta col, 
                $wpdb->postmeta json, 
                $wpdb->posts p 
        WHERE   p.ID = col.post_id AND 
                p.post_status = \"publish\" AND 
                col.post_id = json.post_id AND 
                col.meta_key = \"wpsc_item_collection\" AND 
                col.meta_value = \"$collection_id\" AND 
                json.meta_key = \"wpsc_nft_media_json\" AND 
                json.meta_value LIKE '%\"id\": $media_id%'", ARRAY_A);
        
        if (is_array($res)) {
            return @$res[0]["ID"];
        }

        return false;

    }

}
